#pragma once

#include "FBullCowGame.h"
#include <map>
#include <time.h>

#define TMap std::map // to make sytanx unreal friendly

FBullCowGame::FBullCowGame() { Reset(); } // default constructor

int32 FBullCowGame::GetCurrentTries() const { return MyCurrentTry; }
int32 FBullCowGame::GetHiddenWorldLength() const { return MyHiddenWord.length(); }
bool FBullCowGame::IsGameWon() const { return bGameIsWon; }

void FBullCowGame::Reset()
{
	const int32 MaxWordNumber = 8; // length of our RANDOM_WORDS array, we change this if we want mote items in RANDOM_WORDS array (magic number)
	const Fstring RANDOM_WORDS[MaxWordNumber] = { "crazy", "machine", "music", "plane", "lucky", "right", "lover", "game" }; // words MUST be isogram
	srand(time(NULL)); // set the seed of random to current seconds since January 1, 2000 in the current timezone, otherwise the rand() will allways return same resaults everytime you open the game
	const Fstring HIDDEN_WORD = RANDOM_WORDS[rand() % MaxWordNumber - 1]; // get a random word from the list
	MyHiddenWord = HIDDEN_WORD;

	MyCurrentTry = 1;
	bGameIsWon = false;
	return;
}

int32 FBullCowGame::GetMaxTries() const
{
	TMap<int32, int32> WorldLengthToMaxTries{ {3, 4}, {4, 7}, {5, 10}, {6, 16}, {7, 20} };
	return WorldLengthToMaxTries[MyHiddenWord.length()];
}

EGuessStatus FBullCowGame::CheckGuessValidity(Fstring Guess) const
{
	if (!IsIsogram(Guess)) // if the guess isnt an Isogram
	{ 
		return EGuessStatus::Not_Isogram;
	}
	else if (!IsLowerCase(Guess)) // if the guess isnt all lowercase 
	{
		return EGuessStatus::Not_Lowercase;
	}
	else if (Guess.length() != GetHiddenWorldLength()) // if the guess length is wrong
	{
		return EGuessStatus::Wrong_Length;
	}
	else
	{
		return EGuessStatus::OK;
	}
}

//recieves a vaid guess, incriments turn, and returns count
FBullCowCount FBullCowGame::SubmitValidGuess(Fstring Guess)
{	
	MyCurrentTry++;
	FBullCowCount BullCowCount;
	int32 WordLength = MyHiddenWord.length(); //assuming same length as guest

	// loop through all letters in the hiddenWord
	for(int32 MHWChar = 0; MHWChar < WordLength ; MHWChar++)
	{
		// compare letters against the guess
		for (int32 GChar = 0; GChar < WordLength; GChar++)
		{
			// if they match
			if(Guess[GChar] == MyHiddenWord[MHWChar])
			{
				if (MHWChar == GChar) { // if they are in the same place
					BullCowCount.Bulls++; // increment bulls
				}
				else
				{
					BullCowCount.Cows++; // increment cows
				}
			}
		}
	}
	if (BullCowCount.Bulls == WordLength)
	{
		bGameIsWon = true;
	}
	else
	{
		bGameIsWon = false;
	}

	return BullCowCount;
}


bool FBullCowGame::IsIsogram(Fstring Word) const {
	// treat 0 and 1 letter words asisogram
	if (Word.length() <= 1) { return true; }

	TMap<char, bool> letterSeen; // setup our map
	for (auto letter : Word) // for all the letters of the word
	{
		letter = tolower(letter); // handle mixed case
		if (letterSeen[letter]) { // if the letter is in the map
			return false; // we do not have an isogram	
		}
		else
		{
			letterSeen[letter] = true;// add the letter to the map as seen
		}
	}
	return true; // for example in cases when /0 is entered
}

bool FBullCowGame::IsLowerCase(Fstring Word) const {
	// check if the first letter is not a character
	if (!Word[0] || Word[0] == ' ' || Word[0] == '\\') {
		return false;
	}
	// loop trough all letter is word
	for (auto letter : Word)
	{
		// if the character is upper case
		if (!islower(letter)) {
			return false;
		}
	}
	// if all the above conditions pass then we consider the word as valid lower case
	return true;
}
